package com.yifu.cloud.plus.v1.yifu.insurances.entity;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.yifu.cloud.plus.v1.yifu.common.mybatis.base.BaseEntity;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.hibernate.validator.constraints.Length;

import javax.validation.constraints.NotBlank;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;


/**
 * @author licancan
 * @description 商险明细表 t_insurance_detail
 * @date 2022-07-18 16:20:16
 */
@Data
@EqualsAndHashCode(callSuper = true)
@TableName("t_insurance_detail")
@Tag(name = "商险明细表")
public class TInsuranceDetail extends BaseEntity {
    /**
     * 主键
     */
	@TableId(type = IdType.ASSIGN_ID)
	@Schema(description = "主键")
    private String id;

    /**
     * 员工档案id
     */
    private String empId;

    /**
     * 员工姓名
     */
	@NotBlank(message = "员工姓名不能为空")
	@Length(max = 32, message = "员工姓名不能超过32个字符")
	@Schema(description = "员工姓名")
    private String empName;

    /**
     * 员工身份证号
     */
    private String empIdcardNo;

    /**
     * 订单编号
     */
    private String orderNo;

	/**
	 * 项目编码
	 */
    private String deptNo;

	/**
	 * 投保岗位
	 */
	private String post;

    /**
     * 保险公司名称(冗余字段)
     */
    private String insuranceCompanyName;

    /**
     * 险种主键
     */
    private String insuranceTypeId;

    /**
     * 购买标准
     */
    private String buyStandard;

    /**
     * 保单编号
     */
    private String policyNo;

    /**
     * 保单开始时间
     */
    private LocalDate policyStart;

    /**
     * 保单结束时间
     */
    private LocalDate policyEnd;

    /**
     * 保单生效日期
     */
    private LocalDate policyEffect;

    /**
     * 办理时间
     */
    private LocalDateTime batchDate;

    /**
     * 结算月
     */
    private String settleMonth;

    /**
     * 发票号
     */
    private String invoiceNo;

    /**
     * 购买类型， 1新增、3批增、4替换
     */
    private Integer buyType;

    /**
     * 商险购买地省code
     */
    private Integer insuranceProvince;

    /**
     * 商险购买地市code
     */
    private Integer insuranceCity;

    /**
     * 商险办理省code
     */
    private Integer insuranceHandleProvince;

    /**
     * 商险办理城市code
     */
    private Integer insuranceHandleCity;

    /**
     * 结算类型 (1、单独结算 2、合并结算-和工资一起结算)
     */
    private String settleType;

    /**
     * 实际保费
     */
    private BigDecimal actualPremium;

    /**
     * 预估保费
     */
    private BigDecimal estimatePremium;

    /**
     * 差额（实缴-预估）
     */
    private BigDecimal settleBalance;

    /**
     * 投保办理状态 1待办理 2办理中 3已办理 4投保退回 5 已减员
     */
    private Integer buyHandleStatus;

    /**
     * 结算办理状态 1待结算 2结算中 3已结算
     */
    private Integer settleHandleStatus;

    /**
     * 减员状态 1待减员 2减员中3减员退回
     */
    private Integer reduceHandleStatus;

    /**
     * 是否出险 0未出险 1已出险
     */
    private Integer isUse;

    /**
     * 是否有效 0有效 1无效
     */
    private Integer isEffect;

    /**
     * 是否过期 0未过期 1已过期
     */
	@Schema(description = "是否过期 0未过期 1已过期")
    private Integer isOverdue;

	/**
	 * 备注
	 */
    private String remark;

    /**
     * 是否删除 0未删除/1删除
     */
	@Schema(description = "是否删除 0否/1是")
    private Integer deleteFlag;

    private static final long serialVersionUID = 1L;
}